@@ -1,4 +1,4 @@
-# Copyright 2020 Jared Hendrickson
+# Copyright 2022 Jared Hendrickson
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -11,15 +11,21 @@
 #    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
+"""Defines the client object used to establish virtual pfSense shell connections."""
+__version__ = "2.0.4"
 
-# IMPORT MODULES #
-import requests
-import html
-import urllib3
 import datetime
+import html
+
+import requests
+import urllib3
 
 
 class PFClient:
+    """Client object that facilitates controlling the virtual shell."""
+    # Allow current number of instance attributes, they are needed to allow configurable connections
+    # pylint: disable=too-many-instance-attributes
+
     def __init__(self, host, username, password, port=443, scheme="https", timeout=30, verify=True):
         """
         Initializes the object at creation
@@ -31,6 +37,8 @@ class PFClient:
         :param timeout: (int) the timeout value in seconds for HTTP requests. Defaults to 30.
         :param verify: (bool) true to enable certificate verification, false to disable. Defaults to true.
         """
+        # Allow current number of arguments, it does not affect readability
+        # pylint: disable=too-many-arguments
 
         # Set properties using parameters
         self.session = requests.Session()
@@ -54,7 +62,7 @@ class PFClient:
         Provides the current version of pfsense vShell
         :return: (string) the current pfSense vShell version
         """
-        return "2.0.1"
+        return __version__
 
     def url(self):
         """
@@ -70,10 +78,14 @@ class PFClient:
         :param cmd: (string) a shell command to execute
         :return: (string) output of the shell command
         """
-
+        # Ensure there are no apparent issues with the target host.
         self.__has_host_errors__()
+
+        # Make our HTTP request.
         payload = {"__csrf_magic": self.get_csrf_token("/diag_command.php"), "txtCommand": cmd, "submit": "EXEC"}
         req = self.request("/diag_command.php", method="POST", data=payload)
+
+        # Write the command executed to the vShell history and log the action.
         self.history.append(cmd)
         self.__log__("run_command", cmd)
 
@@ -81,6 +93,9 @@ class PFClient:
         if "<pre>" in req.text:
             return html.unescape(req.text.split("<pre>")[1].split("</pre>")[0])
 
+        # Return none if we were unable to locate the output
+        return None
+
     def request(self, uri, method="GET", data=None):
         """
         Makes HTTP requests on behalf of our object
@@ -92,7 +107,9 @@ class PFClient:
 
         # Try to make our HTTP request, handle errors accordingly
         try:
-            req = self.session.request(method, self.url() + uri, data=data, timeout=self.timeout, verify=self.verify)
+            session = self.session
+            req = session.request(method, self.url() + uri, data=data, timeout=self.timeout, verify=self.verify)
+            session.close()
             self.last_request = req
             self.__log__("request", str(req.status_code) + " " + method + " " + uri)
             return req
@@ -103,16 +120,22 @@ class PFClient:
             self.__log__("request", str(ssl_error))
             self.__get_error__(11)
         except requests.exceptions.ConnectionError as connection_error:
-            self.__log__("request", connection_error)
+            self.__log__("request", str(connection_error))
             self.__get_error__(12)
 
+        # Return none if we somehow land here
+        return None
+
     def authenticate(self):
         """
         Attempts to authenticate using the objects current properties
         :return: (bool) true if authentication was successful, false if it wasn't
         """
 
+        # Make an initial request to the initialize the CSRF checks.
         pre_auth_req = self.request("/index.php")
+
+        # Format our request payload include the valid CSRF token.
         payload = {
             "__csrf_magic": self.get_csrf_token("/index.php"),
             "usernamefld": self.username,
@@ -128,22 +151,24 @@ class PFClient:
             if "username or Password incorrect" not in req.text and "class=\"fa fa-sign-out\"" in req.text:
                 self.__log__("authenticate", "success")
                 return True
-            elif "<p>One moment while the initial setup wizard starts." in req.text:
+            # Support first time logings where wizard is triggered
+            if "<p>One moment while the initial setup wizard starts." in req.text:
                 self.__log__("authenticate", "success")
                 return True
-            else:
-                self.__log__("authenticate", "failed")
-                return False
-        else:
-            return True
+            # Otherwise, assume authentication failed
+            self.__log__("authenticate", "failed")
+            return False
+
+        # Don't re-authenticate if we're already authenticated
+        return True
 
     def get_csrf_token(self, uri):
         """
         Retrieves the current CSRF token for a page
         :param uri: (string) the URI (e.g. index.php) to retrieve the CSRF token from
-        :return: (string) the CSRF token
+        :return: (string) the valid CSRF token or empty string if no CSRF token was found
         """
-
+        # Initialize CSRF token attributes and make initial CSRF query.
         csrf_token_length = 55
         csrf_token = ""
         csrf_resp = self.request(uri, "GET")
@@ -154,7 +179,8 @@ class PFClient:
             csrf += csrf_resp.text.split("sid:")[1].split(";")[0].replace(" ", "").replace("\n", "").replace("\"", "")
             csrf_token = csrf if len(csrf) is csrf_token_length else ""
 
-        return csrf_token  # Return our token
+        # Return the valid CSRF token, or empty string if it could not be determined.
+        return csrf_token
 
     def has_dns_rebind_error(self, req=None):
         """
@@ -162,9 +188,9 @@ class PFClient:
         :param req: (object) optionally provide an existing Response object created by the requests module
         :return: (bool) true if a DNS rebind error was found, false if it wasn't
         """
-
+        # Make a preliminary request to check if a DNS Rebind error was detected by pfSense.
         resp = req.text if req else self.request("/").text
-        return True if "Potential DNS Rebind attack detected" in resp else False
+        return "Potential DNS Rebind attack detected" in resp
 
     def is_host_pfsense(self, req=None):
         """
@@ -173,8 +199,9 @@ class PFClient:
         :param req: (object) optionally provide an existing Response object created by the requests module
         :return: (bool) true if the host is running pfSense, false if it is not
         """
-
+        # Make a preliminary request to check for keywords that indicate the target is running pfSense.
         resp = req.text if req else self.request("/").text
+
         platform_confidence = 0
 
         # List of platform dependent key words to check for
@@ -185,10 +212,10 @@ class PFClient:
             "csrfMagicToken", "/csrf/csrf-magic.js", "wizard.php", "/css/pfSense.css"
         ]
         # Loop through our list and add up a confidence score
-        for ci in check_items:
-            platform_confidence = platform_confidence + 10 if ci in resp else platform_confidence
+        for item in check_items:
+            platform_confidence = platform_confidence + 10 if item in resp else platform_confidence
 
-        return True if platform_confidence > 50 else False
+        return platform_confidence > 50
 
     def __has_host_errors__(self):
         """
@@ -239,10 +266,5 @@ class PFClient:
         self.log.append(",".join([str(datetime.datetime.utcnow()), self.url(), self.username, event, msg]))
 
 
-class PFError(Exception):
-    """
-    Error object used by the PFVShell class
-    """
-    def __init__(self, code, message):
-        self.code = code
-        self.message = message
+class PFError(BaseException):
+    """Error object used by the PFVShell class"""
